import wx
from wx.adv import BitmapComboBox
import wx.lib.agw.supertooltip as SuperToolTip
import flags_iso3166 as flags
try:
    from datetime import datetime, timezone
    from zoneinfo import ZoneInfo               # zoneinfo introduced with python3.9
    tzones = True
except Exception as e:
    tzones = False
'''
flags_iso3166 Dictionary structures:

flags.catalog       iso2 = image
                    iso3 = image        Not for regional entries
                    name = image

flags.country       name = (iso2, is03, regional(True/False)

flags.lookup        iso2 = (iso2, iso3, name, regional(True/False)
                    iso3 = (iso2, iso3, name, regional(True/False)
                    name = (iso2, iso3, name, regional(True/False)

For regional entries in the above, the iso2 entry is a concatenation of country, underscore "_" and region code e.g. US_AL = USA Alabama
 and they have no iso3 code, it is blank

flag.regional       region_iso  = (region_iso, region_name, regional(True))
                    region_name = (region_iso, region_name, regional(True))

flag.details        iso2 = {"continent": continent name,
                            "capital":   city name,
                            "currency":  currency name",
                            "web":       country's Top-Level domain,
                            "lang":      string of language codes separated by | (pipe or vertical bar),
                            "tz":        timezone using UTC standard time offset
                            "altname":   Alternative name (mainly for Russian oblasts)
                           }

flags.languages     language code = language description

flags.timezone      iso2 = iana timezone name


'''


class Myframe(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, -1, flags.__version__, size=(1100, 500))

        self.menubar = wx.MenuBar()
        self.flagscm = wx.Menu()                    # Country flags
        self.flagsrm = wx.Menu()                    # Regional flags

        self.flagsAB = wx.MenuItem(self.flagscm, wx.ID_ANY, 'A to B')
        self.flagsABm = wx.Menu()

        self.flagsUZ = wx.MenuItem(self.flagscm, wx.ID_ANY, 'U to Z')
        self.flagsUZm = wx.Menu()

        self.flagsJapan = wx.MenuItem(self.flagsrm, wx.ID_ANY, 'Japan')
        self.flagsJapanm = wx.Menu()

        self.flagsRussia = wx.MenuItem(self.flagsrm, wx.ID_ANY, 'Russia')
        self.flagsRussiam = wx.Menu()

        self.flagsUSA = wx.MenuItem(self.flagsrm, wx.ID_ANY, 'USA')
        self.flagsUSAm = wx.Menu()

        self.BuildMenus()

        self.menubar.Append(self.flagscm, 'Some Countries')
        self.menubar.Append(self.flagsrm, 'Some Regions')
        self.SetMenuBar(self.menubar)

        panel = wx.Panel(self)
        txt1 = wx.StaticText(panel, -1, 'Name')
        self.combo1 = BitmapComboBox(panel, -1, size=(400, -1), style=wx.TE_PROCESS_ENTER)
        txt2 = wx.StaticText(panel, -1, 'Iso2')
        self.combo2 = BitmapComboBox(panel, -1, style=wx.TE_PROCESS_ENTER)
        txt3 = wx.StaticText(panel, -1, 'Iso3')
        self.combo3 = BitmapComboBox(panel, -1, style=wx.TE_PROCESS_ENTER)
        txt4 = wx.StaticText(panel, -1, 'Regions')
        self.combo4 = BitmapComboBox(panel, -1, size=(400, -1), style=wx.CB_READONLY)

        self.country = wx.StaticText(panel, -1, 'No Country')

        bmp = flags.catalog['Flags'].Bitmap
        self.flag = wx.StaticBitmap(panel, -1, bitmap=bmp)
        sz = bmp.GetSize()
        image = flags.catalog['Flags'].GetImage().Scale(sz[0]*2, sz[1]*2, wx.IMAGE_QUALITY_HIGH)
        bigger = wx.Bitmap(image)
        self.flag_bigger = wx.StaticBitmap(panel, -1, bitmap=bigger)

        self.button = wx.Button(panel, -1, '')
        self.button.SetBackgroundColour('#ffbbff')
        self.button.SetBitmap(bmp)
        self._superTip = SuperToolTip.SuperToolTip("Pirate Flag", bodyImage=bmp, \
                            header="Fun with Flags", headerBmp=bmp, footer=" ", footerBmp=wx.NullBitmap)
        self._superTip.ApplyStyle('Silver Glass')
        self._superTip.SetTarget(self.button)

        txt5 = wx.StaticText(panel, -1, 'X 2')
        txt6 = wx.StaticText(panel, -1, 'Details')
        self.details = wx.TextCtrl(panel, -1, value='', size=(-1, 340), style=wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)

        self.Country_names = sorted(flags.Country_Names)
        self.Country_names.insert(0,"Flags")
        # returns country name if the last data item is False (a NOT a regional entry)
        for i in self.Country_names:
            self.combo1.Append(i, bitmap=flags.catalog[i].Bitmap)

        self.Char2_isocodes = sorted([item for item in flags.catalog.keys() if len(item) == 2])
        self.Char2_isocodes.insert(0,"Flags")
        for i in self.Char2_isocodes:
            self.combo2.Append(i, flags.catalog[i].Bitmap)

        self.Char3_isocodes = sorted([item for item in flags.catalog.keys() if len(item) == 3])
        self.Char3_isocodes.insert(0,"Flags")
        for i in self.Char3_isocodes:
            self.combo3.Append(i, flags.catalog[i].Bitmap)

        self.Regions = sorted(flags.Regional_Names)
        self.Regions.insert(0,"Flags")
        # returns regional name if the last data item is True (is a regional entry)
        for i in self.Regions:
            self.combo4.Append(i, bitmap=flags.catalog[i].Bitmap)

        self.combo1.SetSelection(1)
        self.combo1.Bind(wx.EVT_COMBOBOX, self.OnCombo)
        self.combo1.Bind(wx.EVT_TEXT_ENTER, self.OnCombo)
        self.combo2.SetSelection(1)
        self.combo2.Bind(wx.EVT_COMBOBOX, self.OnCombo)
        self.combo2.Bind(wx.EVT_TEXT_ENTER, self.OnCombo)
        self.combo3.SetSelection(1)
        self.combo3.Bind(wx.EVT_COMBOBOX, self.OnCombo)
        self.combo3.Bind(wx.EVT_TEXT_ENTER, self.OnCombo)
        self.combo4.SetSelection(0)
        self.combo4.Bind(wx.EVT_COMBOBOX, self.OnCombo)
        self.button.Bind(wx.EVT_BUTTON, self.OnButton)

        sizer1 = wx.FlexGridSizer(5, 2, 10, 10)
        sizer1.Add(txt1, 0, wx.EXPAND, border=5)
        sizer1.Add(self.combo1, 0, wx.EXPAND, border=5)
        sizer1.Add(txt2, 0, wx.EXPAND, border=5)
        sizer1.Add(self.combo2, 0, wx.EXPAND, border=5)
        sizer1.Add(txt3, 0, wx.EXPAND, border=5)
        sizer1.Add(self.combo3, 0, wx.EXPAND, border=5)

        sizer2 = wx.FlexGridSizer(3, 2, 10, 10)
        sizer2.AddGrowableCol(1,0)
        sizer2.Add(txt4, 0, wx.EXPAND, border=5)
        sizer2.Add(self.combo4, 1, wx.EXPAND, border=5)
        sizer2.AddSpacer(10)
        sizer2.AddSpacer(10)
        sizer2.Add(txt6, 0, wx.EXPAND, border=5)
        sizer2.Add(self.details, 1, wx.EXPAND, 5)

        flag_sizer = wx.BoxSizer(wx.HORIZONTAL)
        flag_sizer.Add(self.country, 1, wx.ALL, border=5)

        flag_sizer1 = wx.BoxSizer(wx.HORIZONTAL)
        flag_sizer1.Add(self.flag, 0, wx.ALL, border=5)
        flag_sizer1.Add(self.flag_bigger, 0, wx.ALL, border=5)
        flag_sizer1.Add(txt5, 0, wx.ALL|wx.EXPAND, border=5)
        flag_sizer1.Add(self.button, 0, wx.ALL|wx.EXPAND, border=5)

        sizer1.AddSpacer(10)
        sizer1.Add(flag_sizer)
        sizer1.AddSpacer(10)
        sizer1.Add(flag_sizer1)

        main_sizer = wx.BoxSizer(wx.HORIZONTAL)
        main_sizer.Add(sizer1, 0, wx.ALL, 20)
        main_sizer.Add(sizer2, 1, wx.EXPAND|wx.ALL, 20)

        panel.SetSizer(main_sizer)
        panel.Layout()

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(panel, 1, wx.EXPAND, 0)
        self.SetSizer(sizer)
        self.ico = flags.catalog['Flags'].GetIcon()
        self.SetIcon(self.ico)
        self.Layout()

    def OnMenuSelected(self, event):
        id_selected = event.GetId()
        obj = event.GetEventObject()
        select = obj.GetLabel(id_selected)
        self.Reset(select)
        return

    def OnCombo(self, event):
        obj = event.GetEventObject()
        select = event.GetString()
        self.Reset(select)

    def Reset(self, sel):
        # ReSet Combos to match selection and set Region Combo to 1st regional entry if it exists
        try:
            bmp = flags.catalog[sel].Bitmap
        except Exception as e:
            return
        self.flag.SetBitmap(bmp)

        # Scale up a double sized copy of the bitmap
        sz = bmp.GetSize()
        image = flags.catalog[sel].GetImage().Scale(sz[0]*2, sz[1]*2, wx.IMAGE_QUALITY_HIGH)
        bmp2 = wx.Bitmap(image)
        self.flag_bigger.SetBitmap(bmp2)

        # Use the lookup dictionary to get iso2, iso3 and country name e.g. ES ESP Spain 0
        iso2, iso3, name, regional = flags.lookup.get(sel)

        # get selected entry details and store them, as timezone etc may differ from main entry
        # here we assume the selected entry is a regional entry, it may not be
        selected_det = flags.details.get(iso2)
        selected_zone = zone = ''
        if selected_det:
            selected_continent = selected_det.get('continent')
            selected_tz = selected_det.get('tz')
            altname = selected_det.get('altname')
            if tzones:                                  # Were datetime and zoneinfo imports successful
                regzone = flags.timezone.get(iso2)      # zoneinfo introduced with python3.9
                if regzone:
                    selected_zone = regzone
        else:
            selected_continent = selected_tz = altname = ''

        country = name

        # Now if it is regional lookup main entry e.g. JP or US
        # otherwise test for regional entries and set the regional combobox to the first entry for that country
        if regional:
            self.combo4.SetValue(name)
            iso2, iso3, name, regional = flags.lookup.get(iso2[:2])
        else:                                        # Try to find regional entries to reset regional choice
            reg = "Flags"                            # Fall back entry
            region = None
            for region in [r for r in self.Regions if r.startswith(name)]: break
            if region:
                reg = region
            self.combo4.SetSelection(self.Regions.index(reg))

        self.country.SetLabel(country)
        self.combo1.SetSelection(self.Country_names.index(name))
        self.combo2.SetSelection(self.Char2_isocodes.index(iso2))
        self.combo3.SetSelection(self.Char3_isocodes.index(iso3))

        msg =  "Look up:\t\t "+str(flags.lookup.get(sel))+"\n"
        msg += "Regional:\t "+str(flags.regional.get(sel))+"\n"
        msg += "Country:\t\t "+country+"\n"
        det = flags.details.get(iso2)               # Get details of main entry
        if det:
            continent = det.get('continent')
            capital = det.get('capital')
            currency = det.get('currency')
            web = det.get('web')
            tz = det.get('tz')
            lang = det.get('lang')
            if tzones:                              # Were datetime and zoneinfo imports successful
                zone = flags.timezone.get(iso2)     # zoneinfo introduced with python3.9
            else:
                zone = ''

            if selected_continent:
                msg += "Continent:\t "+selected_continent+"\n"
            else:
                msg += "Continent:\t "+continent+"\n"
            msg += "Capital:\t\t "+capital+"\n"
            msg += "Currency:\t "+currency+"\n"
            msg += "Web:\t\t "+web+"\n"
            if altname:
                msg += "Aka:\t\t\t "+altname+"\n"
            if selected_tz:
                msg += "Time Zone:\t "+selected_tz+" UTC standard time\n"
            elif tz:
                msg += "Time Zone:\t "+tz+" UTC standard time\n"
            msg += "Languages:\t "+lang+"\n"
            for l in lang.split("|"):
                l = l.strip().lower()
                if len(l) > 3:
                    l = l[:2]
                lg = flags.language.get(l)
                if lg:
                    msg += ("\n\t\t\t "+lg[0])
        if selected_zone:
            zone = selected_zone
        if zone:
            zone = zone[0]
            timenow = datetime.now(ZoneInfo(zone)).strftime('%H:%M:%S - %d %b %Y')
            localnow = datetime.now().strftime('%H:%M:%S - %d %b %Y')
            if timenow:
                msg += "\n\nLocal time: "+localnow+" - "+zone+": "+timenow
        self.details.SetValue(msg)

        self.button.SetBitmap(bmp)
        self.button.SetName(sel)
        self._superTip.SetBodyImage(bmp)
        self._superTip.SetMessage('Button with the flag of\n'+country)

        self.Layout()

    def OnButton(self, event):
        print("Button Clicked", self.button.GetName())

    def BuildMenus(self):
        # Exclude countries not starting with A or B  |  > 3 whilst avoiding selecting alpha-2 and alpha-3 codes
        x_list = sorted([item for item in flags.catalog.keys() if len(item) > 3 and (item.startswith("A") or item.startswith("B"))])

        for i in x_list:
            iso2, iso3, name, regional  = flags.lookup.get(i)
            if regional:                         # Exclude regions lookup slot 3  is True
                continue
            c1 = wx.MenuItem(self.flagsABm, wx.ID_ANY, i)
            c1.SetBitmap(flags.catalog[i].Bitmap)
            self.flagsABm.Append(c1)
            self.Bind(wx.EVT_MENU, self.OnMenuSelected, id=c1.GetId())
        self.flagsAB.SetSubMenu(self.flagsABm)
        self.flagscm.Append(self.flagsAB)

        allowed = ["U","V","W","X","Y","Z"]
        x_list = sorted(flags.Country_Names)    # Alternative way to get country names

        for i in x_list:
            if i[0] not in allowed:             # Alternative way to exclude keys not starting with U,V,W,X,Y or Z
                continue
            c1 = wx.MenuItem(self.flagsUZm, wx.ID_ANY, i)
            c1.SetBitmap(flags.catalog[i].Bitmap)
            self.flagsUZm.Append(c1)
            self.Bind(wx.EVT_MENU, self.OnMenuSelected, id=c1.GetId())
        self.flagsUZ.SetSubMenu(self.flagsUZm)
        self.flagscm.Append(self.flagsUZ)

        # Build menus for selected regional flags (Japan, Russia and USA)
        x_list = sorted([item for item in flags.regional.keys() if len(item) > 3 and item.startswith("Japan ")])
        for i in x_list:
            c1 = wx.MenuItem(self.flagsJapanm, wx.ID_ANY, i)
            c1.SetBitmap(flags.catalog[i].Bitmap)
            self.flagsJapanm.Append(c1)
            self.Bind(wx.EVT_MENU, self.OnMenuSelected, id=c1.GetId())
        self.flagsJapan.SetSubMenu(self.flagsJapanm)
        self.flagsrm.Append(self.flagsJapan)

        x_list = sorted([item for item in flags.regional.keys() if len(item) > 3 and item.startswith("Russia ")])
        for i in x_list:
            c1 = wx.MenuItem(self.flagsRussiam, wx.ID_ANY, i)
            c1.SetBitmap(flags.catalog[i].Bitmap)
            self.flagsRussiam.Append(c1)
            self.Bind(wx.EVT_MENU, self.OnMenuSelected, id=c1.GetId())
        self.flagsRussia.SetSubMenu(self.flagsRussiam)
        self.flagsrm.Append(self.flagsRussia)

        x_list = sorted(flags.Regional_Names)   # Alternative way to get regional names
        for i in x_list:
            if not i.startswith("United States "):
                continue
            c1 = wx.MenuItem(self.flagsUSAm, wx.ID_ANY, i)
            c1.SetBitmap(flags.catalog[i].Bitmap)
            self.flagsUSAm.Append(c1)
            self.Bind(wx.EVT_MENU, self.OnMenuSelected, id=c1.GetId())
        self.flagsUSA.SetSubMenu(self.flagsUSAm)
        self.flagsrm.Append(self.flagsUSA)

if __name__ == "__main__":
    App = wx.App()
    Myframe().Show()
    App.MainLoop()
