import wx
import flags_iso3166 as flags


class MyFrame(wx.Frame):
    def __init__(self, parent, id, title):
        wx.Frame.__init__(self, parent, -1, title)

        self.tree = wx.TreeCtrl(self, wx.ID_ANY,
                                wx.DefaultPosition,
                                wx.DefaultSize)
        self.tree.SetBackgroundColour('#f1f2f2')
        self.tree.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnTreeCtrl)

        self.choices = {"Australia":"AU","Canada":"CA","Germany":"DE","Spain":"ES","France":"FR",\
                        "United Kingdom":"GB","Italy":"IT","Japan":"JP",\
                        "Russia":"RU","United States of America":"US"}

        self.imagelist = [flags.catalog["Flags"].Bitmap,] # First image for root is Flags


        # Build tree image list
        self.data = sorted([item for item in flags.catalog.keys() if len(item) > 3 and item in self.choices])
        tree_list = []
        for i in self.data:
            subdata = sorted([item for item in flags.catalog.keys() if len(item) > 3 and item.startswith(i)])
            for i in subdata:
                tree_list.append(i)
                iso2, iso3, name, regional = flags.lookup.get(i)
                self.imagelist.append(wx.BitmapBundle.FromBitmap(flags.catalog[iso2].Bitmap))

        self.tree.SetImages(self.imagelist)
        self.root = self.tree.AddRoot('Flags', 0)

        # Add items to Tree
        for idx1, flag in enumerate(tree_list):
            if not flags.isregional(flag):
                x = self.tree.AppendItem(self.root, flag, idx1+1)
            else:
                y = self.tree.AppendItem(x, flag, idx1+1)

        self.tree.Expand(self.root)

        self.Show()

    def OnTreeCtrl(self, event):
        item = event.GetItem()
        id = self.tree.GetItemText(item)
        print(id)


#---------------------------------------------------------------------------

class MyApp(wx.App):
    def OnInit(self):
        frame = MyFrame(None, -1, 'Image TreeCtrl')
        frame.Show(True)
        self.SetTopWindow(frame)

        return True

#---------------------------------------------------------------------------

app = MyApp(0)
app.MainLoop()
